package com.hgwebservice.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.hgwebservice.config.BSXmlWsEntryClass;
import com.hgwebservice.config.BSXmlWsEntryClassService;
import com.hgwebservice.dal.mysql.data.PatientDataMapper;
import com.hgwebservice.entity.hisview.DispatchData;
import com.hgwebservice.entity.hisview.OperatorData;
import com.hgwebservice.entity.hisview.PatientData;
import lombok.extern.slf4j.Slf4j;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
@Service
@Slf4j
public class HgWebServiceImpl implements HgWebService {
    private static SimpleDateFormat SDF_TIME = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private static String url = "http://192.168.200.60:9528/hai/WebServiceEntry";


    @Autowired
    private BSXmlWsEntryClassService service;
    @Autowired
    private PatientDataMapper patientDataMapper;

    @Override
    public List<OperatorData> getOperatorData() {
        List<OperatorData> alloperatorDataList = new ArrayList<>();
        String paramer = "<![CDATA[\n        <BSXml>\n<MsgHeader>\n<Sender>HGXTY</Sender>\n<MsgType>HGXTY_operator_data_xty</MsgType>\n<MsgVersion>3.0</MsgVersion>\n</MsgHeader>\n</BSXml>]]>\n";
        try {
            BSXmlWsEntryClass port = service.getBSXmlWsEntryClassPort();
            String operatorstr = port.invoke("operator_data_xty", "HGXTY","HGXTY",paramer);
            operatorstr = operatorstr.replaceAll("&lt;", "<");
            operatorstr = operatorstr.replaceAll("&gt;", ">");
            if (operatorstr.contains("<Detail>查询成功")) {
                operatorstr = operatorstr.split("</MsgHeader>")[1];
                operatorstr = operatorstr.split("</BSXml>")[0];
                operatorstr = operatorstr.replaceAll(" ", "");
            } else {
//                log.info("get getOperatorDataList Fail =======");
                return alloperatorDataList;
            }
            Document doc = DocumentHelper.parseText(operatorstr);
            Element root = doc.getRootElement();
            List<Element> OperatorList = root.elements();
            for (Element operator : OperatorList) {
                OperatorData operatorData = new OperatorData();
                operatorData.setAcc(operator.element("ACC").getText());
                operatorData.setAccName(operator.element("ACCNAME").getText());
                operatorData.setIsEffective(operator.element("ISEFFECTIVE").getText());
                operatorData.setWardCode(operator.element("WARDCODE").getText());
                operatorData.setWardName(operator.element("WARDNAME").getText());
                operatorData.setDeptCode(operator.element("DEPTCODE").getText());
                operatorData.setDeptName(operator.element("DEPT_NAME").getText());
                alloperatorDataList.add(operatorData);
            }
//            log.info("get getOperatorDataList success =======total:"+alloperatorDataList.size());
            return alloperatorDataList;
        } catch (Exception e) {
            log.error("Get operator info fail ", e);
            return null;
        }
    }

    @Override
    public List<PatientData> getPatientDataByDeptCode(String deptcode) {
        List<PatientData> patientDataList = new ArrayList<>();
//        String paramer = "<![CDATA[\n        <BSXml>\n<MsgHeader>\n<Sender>HGXTY</Sender>\n<MsgType>HGXTY_patient_data_xty</MsgType>\n<MsgVersion>3.0</MsgVersion>\n</MsgHeader>\n<deptcode>" + deptcode + "</deptcode>\n</BSXml>]]>\n";
//        log.info("get PatientUrl paramer ======="+paramer);
        try {
//            BSXmlWsEntryClass port = service.getBSXmlWsEntryClassPort();
//            String patientstr = port.invoke("patient_data_xty", "HGXTY","HGXTY",paramer);
            String paramer = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:ws=\"http://ws.access.hai/\">\n   <soapenv:Header/>\n   <soapenv:Body>\n      <ws:invoke>\n         <service>patient_data_xty</service>\n         <urid>HGXTY</urid>\n         <pwd>HGXTY</pwd>\n     <parameter>\n         <![CDATA[\n        <BSXml>\n<MsgHeader>\n<Sender>HGXTY</Sender>\n<MsgType>HGXTY_patient_data_xty</MsgType>\n<MsgVersion>3.0</MsgVersion>\n</MsgHeader>\n<deptcode>" + deptcode + "</deptcode>\n</BSXml>]]>\n         </parameter>\n      </ws:invoke>\n   </soapenv:Body>\n</soapenv:Envelope>";

            String patientstr = getDataByPost(url, paramer);
            patientstr = patientstr.replaceAll("&lt;", "<");
            patientstr = patientstr.replaceAll("&gt;", ">");
            if (patientstr.contains("<Detail>查询成功")) {
                patientstr = patientstr.split("</MsgHeader>")[1];
                patientstr = patientstr.split("</BSXml>")[0];
            } else {
//                log.info("get PatientUrl Fail ======="+deptcode);
                return patientDataList;
            }
            Document doc = DocumentHelper.parseText(patientstr);
            Element root = doc.getRootElement();
            List<Element> patientList = root.elements();
            for (Element element : patientList) {
                PatientData patientData = new PatientData();
                patientData.setPatientNo(element.element("PATIENT_NO").getText());
                patientData.setHospitalNo(element.element("HOSPITAL_NO").getText());
                patientData.setWristbandNo(element.element("WRISTBAND_NO").getText());
                patientData.setName(element.element("NAME").getText());
                patientData.setHospitalCount(element.element("HOSPITAL_COUNT").getText());
                patientData.setWardCode(element.element("WARD_CODE").getText());
                patientData.setWardName(element.element("WARD_NAME").getText());
                patientData.setDeptCode(element.element("DEPT_CODE").getText());
                patientData.setDeptName(element.element("DEPT_NAME").getText());
                String sex = element.element("GENDER").getText();
                if (!sex.equals("男")) {
                    sex = "女";
                }
                patientData.setGender(sex);
                String birthday = element.element("BIRTHDAY").getText();
                if (birthday.length() < 10){
                    birthday = birthday + " 00:00:00";
                }
                patientData.setBirthday(birthday);
                patientData.setAge(element.element("AGE").getText());
                patientData.setIdentifyNo(element.element("IDENTIFY_NO").getText());
                patientData.setBedNo(element.element("BED_NO").getText());
                String inhosDate = element.element("HOSPITAL_DATE").getText();
                if (inhosDate.length() < 12) {
                    Date date = new Date();
                    inhosDate = SDF_TIME.format(date);
                } else if (inhosDate.length() < 19) {
                    Date date1 = SDF_TIME.parse(inhosDate);
                    inhosDate = SDF_TIME.format(date1);
                }
                patientData.setHospitalDate(inhosDate);
                patientData.setDoctorName(element.element("DOCTOR_NAME").getText());
                patientDataList.add(patientData);
            }
//            log.info(deptcode + "getPatinentList==========>" + patientDataList.size());
            return patientDataList;
        } catch (Exception e) {
            log.error("Get Patientall Fail !!! ", e);
            e.printStackTrace();
            return null;
        }

    }

    private static String getDataByPost(String url, String paramer) {
        String str = "";
        try {
            URL targetUrl = new URL(url);
            HttpURLConnection httpConnection = (HttpURLConnection)targetUrl.openConnection();
            httpConnection.setDoOutput(true);
            httpConnection.setRequestMethod("POST");
            httpConnection.setRequestProperty("Content-Type", "text/xml;charset=utf-8");

            OutputStream outputStream = httpConnection.getOutputStream();
            outputStream.write(paramer.getBytes());
            outputStream.flush();
            if (httpConnection.getResponseCode() != 200)
                throw new RuntimeException("Failed : HTTP error code : " + httpConnection
                        .getResponseCode());
            BufferedReader responseBuffer = new BufferedReader(new InputStreamReader(httpConnection.getInputStream(),"UTF-8"));
            String output;
            while ((output = responseBuffer.readLine()) != null)
                str = str + output;
            httpConnection.disconnect();
        } catch (MalformedURLException e) {
            e.printStackTrace();
            return "0";
        } catch (IOException e) {
            e.printStackTrace();
            return "0";
        }
        return str;
    }

    @Override
    public List<DispatchData> getDispatchDataByDeptCode(String deptcode) {
        List<DispatchData> dispatchDataList = new ArrayList<>();
//        String paramer = "<![CDATA[\n        <BSXml>\n<MsgHeader>\n<Sender>HGXTY</Sender>\n<MsgType>HGXTY_dispatch_data_xty</MsgType>\n<MsgVersion>3.0</MsgVersion>\n</MsgHeader>\n<dept_Code>" + deptcode + "</dept_Code>\n</BSXml>]]>\n";
        try {
//            BSXmlWsEntryClass port = service.getBSXmlWsEntryClassPort();
//            String dispatchstr = port.invoke("dispatch_data_xty", "HGXTY","HGXTY",paramer);
            String paramer = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:ws=\"http://ws.access.hai/\">\n   <soapenv:Header/>\n   <soapenv:Body>\n      <ws:invoke>\n        <service>dispatch_data_xty</service>\n         <urid>HGXTY</urid>\n         <pwd>HGXTY</pwd>\n         <parameter>\n         <![CDATA[\n        <BSXml>\n<MsgHeader>\n<Sender>HGXTY</Sender>\n<MsgType>HGXTY_dispatch_data_xty</MsgType>\n<MsgVersion>3.0</MsgVersion>\n</MsgHeader>\n<dept_Code>" + deptcode + "</dept_Code>\n</BSXml>]]>\n         </parameter>\n      </ws:invoke>\n   </soapenv:Body>\n</soapenv:Envelope>";

            String dispatchstr = getDataByPost(url, paramer);
            dispatchstr = dispatchstr.replaceAll("&lt;", "<");
            dispatchstr = dispatchstr.replaceAll("&gt;", ">");
            if (dispatchstr.contains("<Detail>查询成功")) {
                dispatchstr = dispatchstr.split("</MsgHeader>")[1];
                dispatchstr = dispatchstr.split("</BSXml>")[0];
                dispatchstr = dispatchstr.replaceAll(" ", "");
            } else {
//                log.info("get Dispatch Fail ======="+deptcode);
                return dispatchDataList;
            }
            Document doc = DocumentHelper.parseText(dispatchstr);
            Element root = doc.getRootElement();
            List<Element> dispatchList = root.elements();
            patientDataMapper.selectList(new LambdaQueryWrapper<>());
            for (Element element : dispatchList) {
                DispatchData dispatchData = new DispatchData();
                dispatchData.setDispatchNo(element.element("DISPATCH_NO").getText());
                dispatchData.setType(element.element("TYPE").getText());
                dispatchData.setMessage(element.element("MESSAGE").getText());
                dispatchData.setFrequency(element.element("FREQUENCY").getText());
                dispatchData.setDeptCode(element.element("DEPT_CODE").getText());
                dispatchData.setPatientNo(element.element("PATIENT_NO").getText());
//                dispatchData.setHospitalNo(element.element("HOSPITAL_NO").getText());
//                dispatchData.setDispatchType(element.element("DISPATCH_TYPE").getText());

                dispatchDataList.add(dispatchData);
            }
//            log.info(deptcode + "getdispatchDataList==========>" + dispatchDataList.size());
            return dispatchDataList;
        } catch (Exception e) {
            log.error("Get Dispatch Fail !!! ", e);
            e.printStackTrace();
            return null;
        }
    }
}
